package com.alyseo.serial;

import java.util.*;
import java.util.Map.Entry;
import java.util.regex.*;

import static com.alyseo.serial.CoraidConf.NagiosCodeEnum;

import org.apache.log4j.Logger;


public class CoraidMsgStruct {
	private Logger log = Logger.getLogger("CoraidMsgStruct");

	private Map<String,CoraidElementIntf> coraidComponents = new LinkedHashMap<String,CoraidElementIntf>(); 
	private Map<String,String> whenComponents = new LinkedHashMap<String,String>(); 
	boolean analysed = false;
	private NagiosCodeEnum nagiosCode = NagiosCodeEnum.OK;
	private ArrayList<String> nagiosCriticalMesg =  new ArrayList<String>();
	private ArrayList<String> nagiosWarningMesg = new ArrayList<String>();
	private ArrayList<DiskElement> diskFailed = new ArrayList<DiskElement>();
	private ArrayList<LbladePart> lbladeFailed = new ArrayList<LbladePart>();
	private ArrayList<LbladePart> lbladeDegraded = new ArrayList<LbladePart>();
	private ArrayList<LbladePart> lbladeIniting = new ArrayList<LbladePart>();
	private ArrayList<LbladePart> lbladeNeedInitFailed = new ArrayList<LbladePart>();
	private ArrayList<LbladePart> lbladeNeedInitRecFailed = new ArrayList<LbladePart>();
	private ArrayList<LbladeElement> lbladeOffline = new ArrayList<LbladeElement>();
	
	public void addConf(String conf) {
		CoraidElementIntf cel = MesgFactory.getCoraidImpl(conf);
		if(cel != null) {
			coraidComponents.put(cel.getComponentIndex(), cel);
		}
	}
	public void clear () {
		coraidComponents.clear();
		analysed = false;
		diskFailed.clear();
		lbladeDegraded.clear();
		lbladeFailed.clear();
		lbladeIniting.clear();
		lbladeNeedInitFailed.clear();
		lbladeNeedInitRecFailed.clear();
		lbladeOffline.clear();
		nagiosCriticalMesg.clear();
		nagiosWarningMesg.clear();
		nagiosCode = nagiosCode.OK; 
		whenComponents.clear();
	}
	public void print() {
		Set keys = coraidComponents.entrySet();
		Iterator it = keys.iterator();
		while(it.hasNext()) {
			Entry<String, CoraidElementIntf> entry = (Entry)it.next();
			Object obj = entry.getValue();
			log.debug("Key "+ entry.getKey() + " = " + entry.getValue());
		}
	}
	public void addWhenEntry(String when) {
		Pattern p = Pattern.compile("^(.+)\\s+.+\\s+.+\\s+(\\d+):(\\d+):.+$");
		if( when != null && ! when.equals("") ) {
			when = when.substring(0, when.length()-1);
			Matcher m = p.matcher(when);

			if(m.matches()) {
				whenComponents.put(m.group(1), "(timeleft: " + m.group(2) + ":" + m.group(3) + ")");
				log.debug("when : " + m.group(1) + "/"+m.group(2));
			}
		}
	}
	
	private void analyseConf() {
		log.debug("coraidComponent size : "+ coraidComponents.size());
		for(Entry e : coraidComponents.entrySet()) {
			if(e.getValue() instanceof DiskElementIntf) {
				DiskElement diskElement = (DiskElement)e.getValue();
				if( diskElement.getStatus().equals("failed")) {
					diskFailed.add(diskElement);
					log.fatal("Disk failed : " + diskElement.getDiskNumber());
				}
			}
			if(e.getValue() instanceof LbladePart) {
				LbladePart lbladePart = (LbladePart)e.getValue();
				if( lbladePart.getStatus().equals("failed") || lbladePart.getStatus().startsWith("stop needrecover")) {
					lbladeFailed.add(lbladePart);
					log.fatal("Lblade failed : " + lbladePart.getComponentIndex());
				}
				if( lbladePart.getStatus().startsWith("needrecover recovering degraded") ||
						lbladePart.getStatus().equals("degraded")) {
					lbladeDegraded.add(lbladePart);
					log.warn("Lblade degraded : " + lbladePart.getComponentIndex());
				}
				if( lbladePart.getStatus().startsWith("initing")) {
					lbladeIniting.add(lbladePart);
					log.info("Lblade initing : "+ lbladePart.getComponentIndex());
				}
				if( lbladePart.getStatus().equals("needinit failed")) {
					lbladeNeedInitFailed.add(lbladePart);
					log.fatal("Lblade needinit failed : " + lbladePart.getComponentIndex());
				}
				if( lbladePart.getStatus().equals("needinit needrecover failed")) {
					lbladeNeedInitRecFailed.add(lbladePart);
					log.fatal("Lblade needinit needrecover failed : " + lbladePart.getComponentIndex());
				}
							}
			if(e.getValue() instanceof LbladeElement) {
				LbladeElement lblade = (LbladeElement)e.getValue();
				if(lblade.getStatus().equals("offline")) {
					lbladeOffline.add(lblade);
				}
			}
		}
		// Set Nagios code
		if(diskFailed.size() != 0 ||
				lbladeFailed.size() != 0 ||
				lbladeNeedInitFailed.size() != 0 ||
				lbladeNeedInitRecFailed.size() != 0) {
			setNagiosCode(NagiosCodeEnum.CRITICAL);
			log.fatal("Coraid status : CRITICAL");

		}
		if( (lbladeOffline.size() != 0 || lbladeDegraded.size() != 0) && nagiosCode != NagiosCodeEnum.CRITICAL) {
			setNagiosCode(NagiosCodeEnum.WARNING);
			log.warn("Coraid status : WARNING");
		}
		if(nagiosCode != NagiosCodeEnum.CRITICAL && nagiosCode != NagiosCodeEnum.WARNING) {
			setNagiosCode(NagiosCodeEnum.OK);
			log.info("Coraid status : OK");
		}
		
		// Build Nagios message
		if(diskFailed.size() != 0) {
			// style='background-color:#ff7171'
			addCriticalMessage("<span style='background-color:#ff7171'>Disk Failed :</span> ");
			for(DiskElement diskElement: diskFailed) {
				addCriticalMessage("<b>" + diskElement.getDiskNumber() + "</b>");
				addCriticalMessage(", ");
			}
			nagiosCriticalMesg.remove(nagiosCriticalMesg.size() -1);
			addCriticalMessage("</br>");
		}
		lbladeFailed.addAll(lbladeNeedInitFailed);
		lbladeFailed.addAll(lbladeNeedInitRecFailed);
		if(lbladeFailed.size() != 0) {
			addCriticalMessage("<span style='background-color:#ff7171'>Lblade Failed :</span> ");
			for(LbladePart lbladePart: lbladeFailed) {
				addCriticalMessage("<b>" + lbladePart.getComponentIndex() +"</b>");
				addCriticalMessage(", ");
			}
			nagiosCriticalMesg.remove(nagiosCriticalMesg.size() -1);
			addCriticalMessage("</br>");
		}
		
		lbladeDegraded.addAll(lbladeIniting);
		log.debug("degraded lblades : "+ lbladeDegraded.size());
		if(lbladeDegraded.size() != 0) {
			addWarningMessage("<span style='background-color:#feffc1'>Lblade degraded :</span> ");
			
			for(LbladePart lbladePart: lbladeDegraded) {
				addWarningMessage("<b>" + lbladePart.getComponentIndex());
				if(whenComponents.containsKey(lbladePart.getComponentIndex())) {
					if( ! whenComponents.get(lbladePart.getComponentIndex()).endsWith("0:00)"))
						addWarningMessage(" " + whenComponents.get(lbladePart.getComponentIndex()));
				}
				addWarningMessage(", ");
			}
			nagiosWarningMesg.remove(nagiosWarningMesg.size() -1);
			addWarningMessage("</b></br>");
			
		}
		if(lbladeOffline.size() != 0) {
			for(LbladeElement lblade: lbladeOffline) {
				addWarningMessage("<span style='background-color:#feffc1'>Lblade offline :</span> <b>" + lblade.getComponentIndex() +"</b></br>");
			}
		}
		if(coraidComponents.size() == 0) {
			setNagiosCode(NagiosCodeEnum.WARNING);
			addWarningMessage("<div>No online Lblade</div>");
		}
		
		analysed = true;
	}
	
	private void setNagiosCode(NagiosCodeEnum nagiosCode) {
		this.nagiosCode = nagiosCode;
	}
	
	public void addCriticalMessage(String message) {
		nagiosCriticalMesg.add(message);
	}
	public void addWarningMessage(String message) {
		nagiosWarningMesg.add(message);
	}
	
	protected NagiosCodeEnum getNagiosCode () {
		if( ! analysed) {
			analyseConf();
		}
		return nagiosCode;
	}
	protected String getNagiosMesg() {
		if(!analysed) {
			analyseConf();
		}
		
		log.debug("size of criticalMesg :" + nagiosCriticalMesg.size());
		log.debug("size of warningMesg :" + nagiosWarningMesg.size());
		StringBuffer nagiosMesg = new StringBuffer();
		if(nagiosCriticalMesg.size() != 0) {
			for(String msg : nagiosCriticalMesg) {
				nagiosMesg.append(msg);
				
			}
		}
		if(nagiosWarningMesg.size() != 0) {
			for(String msg : nagiosWarningMesg) {
				nagiosMesg.append(msg);
			}		
		}
		if(nagiosCode == NagiosCodeEnum.OK) {
			nagiosMesg.append("OK");
		}
		String returnMsg = nagiosMesg.toString().replaceAll("</br>$", ""); 
		log.debug("Nagios message size : " + returnMsg.length());
		return nagiosMesg.toString().replaceAll("</br>$", "");
	}
}
