
/**************************************************************************
*   Coraid Monitoring Tool is a interface to communication ports in java
* 	for Coraid products.
*   Copyright 2006 by Alyseo info@alyseo.com
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library; if not, write to the Free
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

package com.alyseo.serial;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.*;

import org.apache.log4j.Logger;

import com.alyseo.util.Configuration;

public class EventsFifo extends Thread {
	
	private List <CoraidConf> list = null;
	private Logger logger = Logger.getLogger("EventsFifo");
	private String[] nsca;
	static private EventsFifo instance = new EventsFifo();
	
	
	static public EventsFifo getInstance() {
		return instance;
	}
	
	public EventsFifo() {
		nsca = new String[7];
		nsca[0] = Configuration.getInstance().getConfigurationFile().getProperty("nsca.bin");
		nsca[1] = "-H";
		nsca[2] = Configuration.getInstance().getConfigurationFile().getProperty("nsca.nagiosServerIP");
		nsca[3] = "-p";
		nsca[4] = Configuration.getInstance().getConfigurationFile().getProperty("nsca.port", "5667");
		nsca[5] = "-c";
		nsca[6] = Configuration.getInstance().getConfigurationFile().getProperty("nsca.conf");
		list = Collections.synchronizedList(new LinkedList<CoraidConf>());
		this.setDaemon(true);
	}

	public synchronized void addMessage(CoraidConf coraidConf) {
		list.add(coraidConf);
		instance.notify();
	}
	
	public synchronized void removeMessage(int number) {
		list.remove(number);
	}
	
	public void run() {
		boolean confOk = true;
		while(true){
			try {
				synchronized(instance) {
					// Wait until thread notified by new message
					instance.wait();
				}
			} catch (InterruptedException ex) {
				logger.fatal("Can not wait for message queue");
			}

			if( nsca[2].matches("^$")) {
				confOk = false;
				logger.fatal("Nagios IP/hostname is empty");
			}
			
			if(! getInstance().list.isEmpty() && confOk){
				while( ! getInstance().list.isEmpty()) {
					logger.debug(getInstance().list.size() + " message to send");
					CoraidConf coraidConf = getInstance().list.get(0);
					try {
						Process child = Runtime.getRuntime().exec(getInstance().nsca);
						Writer writer = new OutputStreamWriter(child.getOutputStream());
						String mesg = coraidConf.getNagiosHostname() + "\t" + 
							coraidConf.getNagiosServiceDesc() + "\t" +
							coraidConf.getNagiosCode() + "\t" +
							coraidConf.getNagiosMessage() + "\n"; 
						writer.write(mesg);
						writer.close();
					} catch (IOException ex) {
						logger.fatal("Can not execute nsca binary (" + ex.getMessage() + ")");
					}
					
					logger.debug("******************** Sending message '" + coraidConf.getNagiosMessage() + "' to nagios server ("+ nsca[2] + ":" + nsca[4] + ")");
					getInstance().removeMessage(0);
				}
			}
		}
	}
}
