
/**************************************************************************
*   Coraid NDM is a interface to communication ports in java
* 	for Coraid products.
*   Copyright 2006 by Alyseo info@alyseo.com
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library; if not, write to the Free
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

package com.alyseo.serial;

import java.io.IOException;
import java.util.*;
import java.util.regex.*;

import org.apache.log4j.Logger;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

import com.alyseo.util.*;

public class Main {
	private final static String version = "1.0.3";
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception{
		Logger logger = Log.getConfiguredLogger("Main");
		logger.info("*************************");
		logger.info("Starting Coraid NDM " + version);
		logger.info("*************************");
		ArrayList portList = Host.getPortsList();
		Iterator it = portList.iterator();
		while(it.hasNext()) {
			CommPortIdentifier port = (CommPortIdentifier)it.next();
			logger.info("Comm port found : " + port.getName());
		}
		logger.debug("Starting fifo thread");
		EventsFifo.getInstance().start();
		
		Properties prop = Configuration.getInstance().getConfigurationFile();
		Properties serial = Configuration.getInstance().getSerialFile();
		if( ! serial.elements().hasMoreElements()){
			logger.error("No serial port configured. Nothing to scan");
		}

		Enumeration e = serial.keys();
		long repeat = 5;
		try {
			repeat = Long.parseLong(prop.getProperty("repeat"));
		} catch (NumberFormatException ex) {
			logger.warn("Unable to parse 'repeat' option in configuration file. Check syntax (eg x minutes). 5 minutes repeat time will be used");
			// do nothing, default value will be used.
		}
		if(repeat == 0){
			repeat = 5;
		}
		Timer timer = new Timer(false);
		CoraidSerialPort c = null;
		while(e.hasMoreElements()) {
			String commPort = (String)e.nextElement();
			String commConf = serial.getProperty(commPort);
			logger.debug(commPort + " = " + serial.getProperty(commPort));
			
			if(Pattern.matches("^(\\d+),(\\d),(\\w),(\\d),(.+?),(.*?),(.+?),(.*)$" , commConf)) {
				CoraidConf coraidConf = new CoraidConf(commPort, commConf);
				logger.debug("Launch thread on port " + coraidConf);
				c = new CoraidSerialPort(coraidConf);
//				timer.scheduleAtFixedRate(c, new Date(), (repeat * 1000));  // seconds
				timer.scheduleAtFixedRate(c, new Date(), (repeat * 1000 * 60)); // minutes
			} else {
				logger.error("Unable to load configuration of port : " + commPort + ". Check syntax please (eg 9600,8,N,1)");
			}
		}
	}
}
